import pandas as pd
import glob
from numpy import array, repeat
import os
import re
from ast import literal_eval


def read_toj(filename):
    """ 
    Reads a simple .toj file from disk .
    :param filename: Path and file to read
    """
    toj = pd.read_table(filename, delimiter=',', header=-1)
    toj = toj.transpose()
    toj.columns = ['SOA', 'Repetitions', 'Count']
    toj['Relative'] = toj['Count'] / toj['Repetitions']
    return toj


def read_tojs(path='.', extension='.toj', look_for_conditions=False):
    """
    Read a directory of .toj files into a single DataFrame
    :param path: Path to files to read
    """
    files = sorted(glob.glob(path+"/*"+extension))
    toj_list = []
    tojs = []
    for i,f in enumerate(files):
        toj_list.append(read_toj(f))
        if look_for_conditions:
            toj_list[-1]['Subject_id']=[int(ff) for ff in [f.split("/")[-1].split('-')[0]]*len(toj_list[-1])]
            condition = f.split(".")[-2].split(".")[-1].split("-")[-1]
            toj_list[-1]['Condition']=[condition]*len(toj_list[-1])
        else:
            toj_list[-1]['Subject_id']=[i]*len(toj_list[-1])

        toj_list[-1]['Filename'] = [f]*len(toj_list[-1])
        tojs = pd.concat(toj_list)
    
    # Conditions found during reading
    if look_for_conditions:   
        # Add condition numbers:
        condition_numbers = []
        if "Condition" in tojs.keys():
            conditions = sorted(list(set(tojs['Condition']))) 
            for t in tojs.iterrows():
                condition_numbers.append(conditions.index(t[1]['Condition']))
        tojs['Condition_num'] = condition_numbers
    
    tojs = update_running_subject_numbers(tojs)
    return tojs
        
    #tojs = pd.concat([ ])
   # tojs.index = range(0,len(tojs))
   
def read_tojs_conditions_within_file(path='.', extension='.toj'):
    """
    Read a directory of .csv files into a single DataFrame
    :param path: Path to files to read
    """
    files = sorted(glob.glob(path+"/*"+extension))
    toj_list = []
    tojs = []
    for i,f in enumerate(files):
        temp = pd.read_csv(f)
        toj_list.append(temp)
        toj_list[-1]['Subject_id']=[i]*len(toj_list[-1])
        toj_list[-1]['Filename'] = [f]*len(toj_list[-1])
        tojs = pd.concat(toj_list)
    tojs = update_running_subject_numbers(tojs)
    tojs = update_running_condition_numbers(tojs)
    return tojs



def read_letter_toj(filename, name_SOA='SOA', names_shown=('probeLetter',
                    'refLetter'), names_reported=('firstResp',
                    'secondResp'), **kwargs):
    
    toj_raw = pd.read_table(filename, **kwargs)
    
    # Add a probe first column
    toj_raw['Probe first'] = ((toj_raw[names_shown[0]] == toj_raw[names_reported[0]]) |
                              (toj_raw[names_shown[1]] == toj_raw[names_reported[1]]))
    
    # Squeeze the counts out of the data:
    SOAs = toj_raw[name_SOA].value_counts().sort_index().index.values
    pfs = toj_raw[name_SOA][toj_raw['Probe first'] == True]

    pfc = pfs.value_counts(dropna=False).sort_index()
    reps = toj_raw[name_SOA].value_counts().sort_index()
    pfc = [pfc[x] if x in pfc else 0 for x in SOAs]
    toj = pd.DataFrame(array([SOAs,pfc,reps,pfc/reps]).T)
    #toj.columns = ['SOA','Repetitions','Count','Relative'] 
    #TODO: Remove if line below is in correct order
    toj.columns = ['SOA','Count', 'Repetitions','Relative'] 
    return toj


def read_letter_tojs(path='.', extension='.toj', 
                     pattern='(?:subject-)([0-9]+)(?:.csv)',
                     look_for_conditions=False, name_SOA='SOA',
                     names_shown=('probeLetter','refLetter'), 
                     names_reported=('firstResp', 'secondResp'), **kwargs):
    
    if look_for_conditions:
        raise NotImplementedError('Currently not implemented!')
    
    # Get the file names
    files = sorted([f for f in os.listdir(path) if re.match(pattern,f)])
    
    # Get the subject ids
    subject_ids = [int(re.match(pattern,f).groups()[0]) for f in files]
    
    # Read TOJs in to a list
    toj_list = [read_letter_toj(path + '/'+ f,name_SOA, names_shown,names_reported,**kwargs) for f in files]
    
    # Insert subject ids into TOJs
    for i, t in enumerate(toj_list):
        t['Subject_id'] = repeat(subject_ids[i], len(t))
        t['Running_subject_number'] = repeat(i, len(t))

    return pd.concat(toj_list).reset_index()

def read_psylab_toj_stimulus_toj(filename, condition_col=None, condition_names=None, **kwargs):
    """
    Read a CSV produced by the OpenSesame plugin psylab_toj_stimulus into a DataFrame.
    :param condition_names: A list of all possible condition names.
    :param condition_col: This only works if there's a column with the name of the condition already in the file.
        If that is not the case, you will either need to prepare the file before calling this function,
        or you will have to write your own function for reading the file into a toj DataFrame.
    :param filename: Filename of the CSV.
    :return:
    """
    
    name_soa = 'TOJ_soa'
    toj_raw = pd.read_table(filename, **kwargs)

    # Add a probe first column
    for index, row in toj_raw.iterrows():
        resp = literal_eval(row['TOJ_letter_response'])  # convert string like "[u'R', u'S']" to list
        targets = literal_eval(row['TOJ_targets'])
        probe_first = (len(resp) > 0 and (resp[0] == targets[0])) or \
                      (len(resp) > 1 and (resp[1] == targets[1]))
        toj_raw.set_value(index, 'Probe first', probe_first)

   
    # Squeeze the counts out of the data:
    toj_list = []
    
    if condition_col != None:
        
         # Add condition column
        toj_raw['Condition'] = toj_raw[condition_col]
        for index, row in toj_raw.iterrows():
            toj_raw.set_value(index, 'Condition_num', condition_names.index(row[condition_col]))


        for condition_name in condition_names:
            cond_index = condition_names.index(condition_name)
            SOAs = toj_raw[name_soa][toj_raw["Condition_num"] == cond_index].value_counts().sort_index().index.values
            pfs = toj_raw[name_soa][(toj_raw['Probe first'] == True) & (toj_raw["Condition_num"] == cond_index)]  # probe firsts
    
            pfc = pfs.value_counts(dropna=False).sort_index()  # probe first count
            reps = toj_raw[name_soa][toj_raw["Condition_num"] == cond_index].value_counts().sort_index()
            pfc = [pfc[x] if x in pfc else 0 for x in SOAs]
            toj = pd.DataFrame(array([SOAs, pfc, reps, pfc / reps]).T)
            toj.columns = ['SOA', 'Count', 'Repetitions', 'Relative']
            toj['Condition'] = condition_name
            toj['Condition_num'] = cond_index
            toj_list.append(toj)
    else:

        SOAs = toj_raw[name_soa].value_counts().sort_index().index.values
        pfs = toj_raw[name_soa][(toj_raw['Probe first'] == True)]   # probe firsts

        pfc = pfs.value_counts(dropna=False).sort_index()  # probe first count
        reps = toj_raw[name_soa].value_counts().sort_index()
        pfc = [pfc[x] if x in pfc else 0 for x in SOAs]
        toj = pd.DataFrame(array([SOAs, pfc, reps, pfc / reps]).T)
        toj.columns = ['SOA', 'Count', 'Repetitions', 'Relative']
        toj['Condition'] = repeat('',len(toj))
        toj['Condition_num'] = repeat(0,len(toj))
        toj_list.append(toj)
        print(toj)
    return pd.concat(toj_list)


def read_psylab_toj_stimulus_tojs(path='.', condition_names=None, condition_col=None,
                                  pattern='(?:subject-)([0-9]+)(?:.csv)',
                                  **kwargs):
    # Get the file names
    files = sorted([f for f in os.listdir(path) if re.match(pattern, f)])
    
    print(files)

    # Get the subject ids
    subject_ids = [int(re.match(pattern, f).groups()[0]) for f in files]
    print("about to call")
    # Read TOJs in to a list
    toj_list = [read_psylab_toj_stimulus_toj(path + '/' + f, condition_col, condition_names, **kwargs) for f in files]
    print("after call")

    # Insert subject ids into TOJs
    for i, t in enumerate(toj_list):
        t['Subject_id'] = repeat(subject_ids[i], len(t))
        t['Running_subject_number'] = repeat(i, len(t))

    return pd.concat(toj_list).reset_index()

def exclude(tojs_in, subject_ids):
    """
    Takes a TOJ DataFrame and excludes data based on the subject ids.
    :param tojs_in:     Original TOJ DataFrame
    :param subject_ids: List of subject ids to be excluded from the returned
                        DataFrame
    :return: tojs_out:  New data frame of the remaining TOJs                
    """
    tojs_out = tojs_in.loc[~tojs_in['Subject_id'].isin(subject_ids)]
    tojs_out = update_running_subject_numbers(tojs_out)
    return tojs_out


def include(tojs_in, subject_ids):
    """
    Takes a TOJ DataFrame and excludes data based on the subject ids.
    :param tojs_in:     Original TOJ DataFrame
    :param subject_ids: List of subject ids to be included in the returned
                        DataFrame
    :return: tojs_out:  New data frame of the remaining TOJs                
    """
    tojs_out = tojs_in.loc[tojs_in['Subject_id'].isin(subject_ids)]
    tojs_out = update_running_subject_numbers(tojs_out)
    return tojs_out


def update_running_subject_numbers(tojs):
    """
    """
    tojs['Running_subject_number'] = [list(set(tojs['Subject_id'])).index(t) for t in tojs['Subject_id']]
    return tojs

def update_running_condition_numbers(tojs):
    """
    """
    tojs['Condition_num'] = [list(set(tojs['Condition'])).index(t) for t in tojs['Condition']]
    return tojs
