import pymc3
import tvatoj
from numpy import repeat, array
from scipy import optimize
from tvatoj.models.base_model import BaseModel
from theano.tensor import mean

class AQGPBaseModel(BaseModel):
    """This is the base class for TOJ models based on two TVA rates."""
    
    def lambdap_rate(self):
        """Override this function to produce the probe rate """
        with self.model:
            self.lambdap = pymc3.Uniform('lambdap',0,
                                    self.default_uniform_probe_prior_limit, 
                                    shape=(self.nidx, self.ncnd)) 
        
    def lambdar_rate(self):
        """Override this function to produce the reference rate """
        with self.model:
            self.lambdar = pymc3.Uniform('lambdar', 0,
                                    self.default_uniform_reference_prior_limit,
                                    shape=(self.nidx, self.ncnd))  
            
    def delta_time(self):
        """Override this function to produce the delta time """
        with self.model:
            self.delta = pymc3.Uniform('delta', 0,
                                    self.default_uniform_delta_prior_limit,
                                    shape=(self.nidx, self.ncnd))  
            
    def tau_time(self):
        """Override this function to produce the tau time  """
        with self.model:
            self.tau = pymc3.Uniform('tau', 
                                    -self.default_uniform_tau_prior_limit,
                                    +self.default_uniform_tau_prior_limit,
                                    shape=(self.nidx, self.ncnd)) 
            
    def xi_bias(self):
        """Override this function to produce the response bias xi  """
        with self.model:
            self.xi = pymc3.Uniform('xi', self.default_uniform_xi_prior_limit,
                                    1 - self.default_uniform_xi_prior_limit,
                                    shape=(self.nidx, self.ncnd)) 
            
    def epsilonp_error(self):
        """Override this function to produce the epsilon probe error """
        with self.model:
            self.epsilonp = pymc3.Uniform('epsilonp', 0,
                                    self.default_uniform_epsilonp_prior_limit,
                                    shape=(self.nidx, self.ncnd)) 
            
    def epsilonr_error(self):
        """Override this function to produce the epsilon reference error """
        with self.model:
            self.epsilonr = pymc3.Uniform('epsilonr', 0,
                                    self.default_uniform_epsilonr_prior_limit,
                                    shape=(self.nidx, self.ncnd)) 
 
    def __init__(self, tojs):
        """
        @param tojs: A TVATOJ dataframe.
        """

        #AQGPBaseModel.__init__(self, tojs)
        #super(BaseModel, self).__init__(tojs)
        BaseModel.__init__(self, tojs)



        with self.model:
       

            self.default_uniform_probe_prior_limit = 0.500
            self.default_uniform_reference_prior_limit = 0.500
            self.default_uniform_delta_prior_limit = 100
            self.default_uniform_tau_prior_limit = 100
            self.default_uniform_xi_prior_limit = 0.4
            self.default_uniform_epsilonp_prior_limit = 0.5
            self.default_uniform_epsilonr_prior_limit = 0.5


            self.lambdap_rate()
            self.lambdar_rate()
            self.delta_time()
            self.tau_time()
            self.xi_bias()
            self.epsilonp_error()
            self.epsilonr_error()
            
            # Out deterministic model function to generate the success rates at the SOAs:
            v_diff = pymc3.Deterministic('lambda_diff', self.lambdap - self.lambdar)
            v_diff_mean = pymc3.Deterministic('lambda_diff_mean', mean(v_diff))
            theta = pymc3.Deterministic('theta', tvatoj.pf_aqqp_toj(self.SOA, 
                                                                   self.lambdap[(self.idx,self.cnd)], 
                                                                   self.lambdar[(self.idx,self.cnd)],
                                                                   self.delta[(self.idx,self.cnd)],
                                                                   self.tau[(self.idx,self.cnd)],
                                                                   self.xi[(self.idx,self.cnd)],
                                                                   self.epsilonp[(self.idx,self.cnd)],
                                                                   self.epsilonr[(self.idx,self.cnd)]
                                                                   ))
            # Now link it to the observation with a binomial likelihood:
            y = pymc3.Binomial('y', n = self.reps, p = theta, 
                                observed=self.probe_first_count, shape=(len(self.idx)))
            
            

    
    
