import pymc3
import tvatoj
from numpy import repeat
from scipy import optimize
from theano.tensor import mean

class BaseModel():
    """This is the base class for TOJ models based on two TVA rates."""
    def __init__(self, tojs):
        """
        @param tojs: A TVATOJ dataframe.
        """
        self.default_uniform_probe_prior_limit = 0.050 # TODO: previously 0.200 which excluded less rates - why is 0.050 better? AK
        self.default_uniform_reference_prior_limit = 0.050
        self.model =  pymc3.Model()
        self.tojs = tojs
        with self.model:
            self.idx = tojs.Running_subject_number.get_values()
            self.nidx = tojs.Running_subject_number.nunique()
            self.SOA = tojs.SOA.get_values()
            self.reps = tojs.Repetitions.get_values()
            self.probe_first_count = tojs.Count.get_values()
            
            # if information on conditions are present in the dataframe
            if 'Condition_num' in tojs.columns.values.tolist() and \
                'Condition' in tojs.columns.values.tolist():
                self.cnd = tojs.Condition_num.get_values()
                self.ncnd = tojs.Condition_num.nunique()
            # if no information on condotion, treat as if there was only one condition
            else:
                self.cnd = repeat(0,len(self.idx))
                self.ncnd = 1
                
    def sample(self, samples=2000,njobs=4, thin=1, **kwargs):
        """
        @param samples: Number of MCMC samples.
        @param samples: njobs of parallel MCMC chains.
        """
        with self.model:
            tvatoj.tic('Sampling')
            # Four parallel chains, 20000 samples
            start = pymc3.find_MAP(fmin=optimize.fmin_powell)
            # debug('These are the starting values: ' +str(start))
            step = pymc3.NUTS(state=start)
            self.trace = pymc3.sample(samples, step, progressbar=True, njobs=njobs, start=start, **kwargs) 
            if thin > 1:
                self.trace = self.trace[::thin]
            tvatoj.toc()
    def traceplot(self):
        with self.model:
            pymc3.traceplot(self.trace)
            
    def waic(self):
        pymc3.waic(model=self.model, trace=self.trace)
            
    def dfsummary(self, file=None):
        with self.model:
            if file == None:
                return pymc3.stats.df_summary(self.trace)
            else:
                pymc3.stats.df_summary(self.trace).to_csv(file)

