import pymc3
import tvatoj
from numpy import repeat
from scipy import optimize
from tvatoj.models.base_model import BaseModel
from theano.tensor import mean
from tvatoj.helpers.helpers import info

class TVABaseModel(BaseModel):
    """This is the base class for TOJ models based on two TVA rates."""
    
    def vp_rate(self):
        """Override this function to produce the probe rate """
        info("Probe Rate setup in TVABaseModel")
        with self.model:
            self.vp = pymc3.Uniform('vp',0,
                                    self.default_uniform_probe_prior_limit, 
                                    shape=(self.nidx, self.ncnd)) 
        
    def vr_rate(self):
        """Override this function to produce the reference rate """
        info("Reference Rate setup in TVABaseModel")
        with self.model:
            self.vr = pymc3.Uniform('vr', 0,
                                    self.default_uniform_reference_prior_limit,
                                    shape=(self.nidx, self.ncnd))  
    def derived_estimates(self):
        info("Setting derived estimates from TVABaseModel")
        with self.model:
            self.v_diff = pymc3.Deterministic('v_diff', self.vp - self.vr)
            self.v_diff_mean = pymc3.Deterministic('v_diff_mean', mean(self.v_diff, axis=0))
            self.C = pymc3.Deterministic('C', self.vp + self.vr)
            self.w_p = pymc3.Deterministic('w_p', self.vp / (self.vp + self.vr) ) 
        
    def __init__(self, tojs):
        """
        @param tojs: A TVATOJ dataframe.
        """
        BaseModel.__init__(self, tojs)
        with self.model:
            self.default_uniform_probe_prior_limit = 0.200
            self.default_uniform_reference_prior_limit = 0.200
            
            self.vp_rate()
            self.vr_rate()
            self.derived_estimates()   
            
            # Out deterministic model function to generate the success rates at the SOAs:
            theta = pymc3.Deterministic('theta', tvatoj.pf_tva_simple_toj(self.SOA, 
                                                                   self.vp[(self.idx,self.cnd)], 
                                                                   self.vr[(self.idx,self.cnd)]))
            # Now link it to the observation with a binomial likelihood:
            y = pymc3.Binomial('y', n = self.reps, p = theta, 
                                observed=self.probe_first_count, shape=(len(self.idx)))

