from numpy import array, abs, exp, log, power

def pf_logistic(SOA, m, w):
    '''
    Logistic psychometric funtion from Kuss et al. 2005
    a = 0.1
    :param SOA: Vector containing the SOAs [ms].
    :param m: threshold (m = F^(-1)(0.5)).
    :param w: width (w = F^(-1)(1-a)-F^(-1)(a)).
    :returns: Vector of relative frequency of 'probe first' judgments.  
    '''
    a = 0.1
    za = 2*log(1/(a-1))
    return power( 1+exp( -(za/w)*(SOA-m) ), -1)


def pf_tva_simple_toj(SOA, vp, vr):
    '''
    Simple TVA-based TOJ model. 
    :param SOA: Vector containing the SOAs [ms].
    :param vp: Rate of the probe target (v_p).
    :param vr: Rate of the reference target (v_r).
    :returns: Vector of relative frequency of 'probe first' judgments.    
    '''
    SOA = array(SOA)
    left = (1-exp(-vp*abs(SOA))) + exp(-vp*abs(SOA)) * vp/(vp+vr)
    right = exp(-vr*abs(SOA))*vp/(vp+vr)
    return ((SOA<=0)*left  + (SOA>0)*right)


def pf_tva_simple_toj_2(SOA, vp, vr):
    '''
    Simple TVA-based TOJ model. Alternative version with one less exp. 
    :param SOA: Vector containing the SOAs [ms].
    :param vp: Rate of the probe target (v_p).
    :param vr: Rate of the reference target (v_r).
    :returns: Vector of relative frequency of 'probe first' judgments.    
    '''
    SOA = array(SOA)
    left =  1-(exp(-vp*abs(SOA)) * vr/(vp+vr))
    right = exp(-vr*abs(SOA))*vp/(vp+vr)
    return ((SOA<=0)*left  + (SOA>0)*right)

def pf_aqqp_toj(SOA, lambdap, lambdar, delta, tau, xi):
    '''
    Simple TVA-based TOJ model. 
    : TODO
    :returns: Vector of relative frequency of 'probe first' judgments.    
    '''
    SOA = array(SOA)
    pPF = pf_tva_simple_toj(SOA + tau - delta, lambdap, lambdar)
    pRF = 1 - pf_tva_simple_toj(SOA + tau + delta, lambdap, lambdar)
    pS  = 1 - pPF - pRF
    p   = (1 * pPF + (1-xi)*pS + 0 * pRF)
    return p
    

def pf_tva_toj_with_cue(SOA, vp, vr, vc, COA):
        '''
        Model for cued TOJs (experimental)
        :param SOA: Vector containing the SOAs [ms].
        :param vp: Rate of the probe target (v_p).
        :param vr: Rate of the reference target (v_r).
        :param vc: Rate of cue-probe confusions.
        :param COA: Cue onset asyncrhony [ms].        
        :returns: Vector of relative frequency of 'probe first' judgments.    
        '''
        SOA = array(SOA)
        left = 1-exp(-vc*COA) + ((exp((-vc)*(COA))) ) * ((1-exp((-vp)*abs(SOA))) + (exp((-vp)*abs(SOA))) *  (vp / (vp + vr)))
        middle = 1-exp(-vc*(COA-SOA)) + ((exp((-vc)*(COA-SOA)))) * ( vc*(1/(vc+vr)-exp(-vc*SOA-vr*SOA)/(vc+vr)) +((exp((-vc)*(SOA))) * (exp((-vr)*(SOA)))) * (vp / (vp + vr)))
        right =   exp(  -((vr+vp+vc)/2)   *(SOA-COA)) * (vc*(1/(vc+vr)-exp(-vc*COA-vr*COA)/(vc+vr)) + ((exp((-vc)*(COA))) * (exp((-vr)*(COA)))) * (vp / (vp + vr)))   
        return ((SOA<=0) * left  + ((SOA > 0) * (SOA <= COA)) * middle + ((SOA > 0) * (SOA > COA)) * right)
