'''
@author: jeti

This module contains simulations of TOJs. Note that they are written in an non-
vectorized and potentially inefficient form. The goal is to facilitate 
understanding of the generative processes. 

'''


from numpy.random import exponential
from numpy import array
import pandas

def pf_tva_simple_toj_sim(SOAs, vp, vr, repetitions):
    '''
    Simulation of "probe first" perceptions in a simple binary TOJ.
    :param SOAs: Vector of the SOAs.
    :param vp: Rate of the probe stimulus.
    :param vr: Rate of the reference stimulus.
    :param repetitions: Vector of the repetitions in each SOA.
    :returns: A vector of "probe first" counts per SOA.
    '''
    
    probe_first_count = []
    
    for i, SOA in enumerate(SOAs):
        
        probe_first_count.append(0)
        
        # For every repetition of the current SOA
        for _ in range(0,repetitions[i]):    
            
            # Get a random exponential encoding duration for the reference and ...    
            tr = exponential(1/vr)
            # ... for the probe. Add the SOA as a fixed delay between the two.
            tp =  SOA + exponential(1/vp)
           
            # If the probe is encoded before the reference, increase the counter
            if tp < tr:
                probe_first_count[-1] = probe_first_count[-1] + 1 
                
    # Construct a TOJ data frame  
    toj = pandas.DataFrame([list(SOAs),list(repetitions),probe_first_count,
                             list(array(probe_first_count,dtype=float) / 
                                  repetitions)]).T
    toj.columns = ['SOA', 'Repetitions', 'Count', 'Relative']
    
    return toj




def pf_tva_toj_with_cue_unmasked_sim(SOAs, vp, vr, vcp, repetitions, COA):
    '''
    Simulation of "probe first" perceptions in a simple binary TOJ.
    
    In this version, the target does not mask the cue!
    
    :param SOAs: Vector of the SOAs.
    :param vp: Rate of the probe stimulus.
    :param vr: Rate of the reference stimulus.
    :param vcp: Rate of cue--probe confusion
    :param repetitions: Vector of the repetitions in each SOA.
    :param COA: The cue onset asynchrony (a.k.a cueing interval)
    :returns: A vector of "probe first" counts per SOA.
    '''
    
    probe_first_count = []
    
    for i, SOA in enumerate(SOAs):
        
        probe_first_count.append(0)
        
        
        # For every repetition of the current SOA
        for _ in range(0,repetitions[i]):    
            
            # Get a random exponential encoding duration for the reference and ...    
            tr = exponential(1/vr)
            
            # ... for the probe. Add the SOA as a fixed delay between the two.
            tp =  SOA + exponential(1/vp)
            
            # Get a time for the cue probe confusion
            tcp =  -COA + SOA + exponential(1/vcp)
           
            # If the probe is encoded before the reference, increase the counter
            if tp < tr or tcp < tr:
                probe_first_count[-1] = probe_first_count[-1] + 1 
                
    # Construct a TOJ data frame  
    toj = pandas.DataFrame([list(SOAs),list(repetitions),probe_first_count,
                             list(array(probe_first_count,dtype=float) / 
                                  repetitions)]).T
    toj.columns = ['SOA', 'Repetitions', 'Count', 'Relative']
    
    return toj



def pf_tva_toj_with_cue_sim(SOAs, vp, vr, vcp, repetitions, COA, 
                            no_precognition=True):
    '''
    Simulation of "probe first" perceptions in a simple binary TOJ.
    :param SOAs: Vector of the SOAs.
    :param vp: Rate of the probe stimulus.
    :param vr: Rate of the reference stimulus.
    :param vcp: Rate of cue--probe confusion
    :param repetitions: Vector of the repetitions in each SOA.
    :param COA: The cue onset asynchrony (a.k.a cueing interval)
    :param no_precognition: Should be True. If set to false, the visual system
                            would know in advance where the cue will appear.
    :returns: A vector of "probe first" counts per SOA.
    '''
    
    probe_first_count = []
    
    for i, SOA in enumerate(SOAs):
        
        probe_first_count.append(0)
        
        # For every repetition of the current SOA
        for _ in range(0,repetitions[i]):    
                    
            
            # For negative SOAs the cue is the first stimulus, followed by the  
            # probe:
            # ----C---<COA>---P----------------
            # ----------------|---<SOA>---R----
            if SOA < 0:
                
                # Get a time for the cue probe confusion
                tcp = exponential(1/vcp)
                
                # If the cue is as probe encoded before the probe appears ...
                if tcp < COA: 
                    
                    # That's a probe first event
                    probe_first_count[-1] = probe_first_count[-1] + 1 
                    
                else:
                    
                    # Get a random exponential encoding durations for probe and 
                    # reference as for simple TOJs
                    tr = exponential(1/vr)
                    tp =  SOA + exponential(1/vp)
                    
                    # If the probe is encoded before the reference
                    if tp < tr:
                        # That's a probe first event
                        probe_first_count[-1] = probe_first_count[-1] + 1 
                        
            # If the SOA is larger than zero, the order can be
            # cue->reference->probe or reference->cue->probe.           
            else: # SOA >= 0
                
                # Order cue->reference->probe:
                # -----C-----<COA>----P-------
                # ----------R--<SOA>--|-------
                if COA>SOA:
                    
                    # Get a time for the cue probe confusion
                    tcp =  exponential(1/vcp)
                    if tcp < (COA-SOA):
                        # That's a probe first event
                        probe_first_count[-1] = probe_first_count[-1] + 1 
                    else:
                        tcp =  exponential(1/vcp)
                        tr =  exponential(1/vr)
                        if tcp < SOA or tcp < SOA: # Both en the TOJ here
                            if tcp < tr:
                                # That's a probe first event
                                probe_first_count[-1] = probe_first_count[-1] + 1 
                        else:
                            tp =  exponential(1/vp)
                            tr =  exponential(1/vr)
                            if tp < tr-SOA:
                                    # That's a probe first event
                                    probe_first_count[-1] = probe_first_count[-1] + 1 
                            
                # Order reference->cue->probe:
                # ---------C---<COA>---P-------
                # ---R-----<SOA>-------|------- 
                       
                else: # COA<=SOA  
                    
                    # If the reference come first, attention has not yet been altered
                    # It should therefore be assumed that half the resources are spent
                    # for the first target (in anticipation of the second)
                    if no_precognition:
                        tr =  exponential(1/((vr+vp+vcp)/2.0))
                    else:
                        tr =  exponential(1/vr)
                    
                    
                    if tr < (SOA-COA):
                        # Nothing happens!
                        True 
                    else:
                        tcp =  exponential(1/vcp)
                        tr =  exponential(1/vr)
                        if (tcp<COA) or (tr<COA):
                            if(tcp<tr):
                                probe_first_count[-1] = probe_first_count[-1] + 1
                        else:
                            tp =  exponential(1/vp)
                            tr =  exponential(1/vr)
                            if (tp < tr):
                                probe_first_count[-1] = probe_first_count[-1] + 1
       
    # Construct a TOJ data frame  
    toj = pandas.DataFrame([list(SOAs),list(repetitions),probe_first_count,
                             list(array(probe_first_count,dtype=float) / 
                                  repetitions)]).T
    toj.columns = ['SOA', 'Repetitions', 'Count', 'Relative']
    
    return toj
        
