from tvatoj import est_mode
from numpy import array, float64


def rate_label(values,rate_names,var_names=[], unit='Hz', factor=1, fmt="%2.3f",
                 condition=''):

    ''' 
    This creates a label for a plot that includes the estimates of (multiple)
    rates. 
        
    :param values:  List of values or pymc3 trace. If a trace is provided, the 
                    modes are estimated and used as values for the rates. Then, 
                    var_names must also be specified.
    :param rate_names: List of human readable names of the rates in the same 
                    order as the values (or var_names). Should be in LaTeX math
                    syntax (e.g., ['v_p^{\mu}','v_r^{\mu}']).
    :param var_names: If a trace is used, this list specifies the parameter 
                    names of the rates in the trace.
    :param unit:    A string representing the unit, e.g. 'Hz'.
    :param factor:  A factor the values are multiplied with. E.g., 1000 for the
                    "ms/Hz" scale. TODO: Link and document ms/Hz.
    :param fmt:     Format specifier.
    :param condition: String identifying the experimental condition.
    
    :returns:       Returns a tuple. First element is a string which can
                    be used as a plot label.  The second element is a list
                    of the values (that might have been estimated).
                    E
                    .g.   
                    >>> rate_labels((60.0,45.0),('v_p','v_r'),condition='Cue',
                    fmt="%2.2f")
                    ('Cue\n  v_p = 60.00$ Hz\n  v_r = 45.00$ Hz', (60.0,45.0))
                
    :raise Exception: If a trace is provided and no var_names are specified, an
                    exception is raised.

    '''
    label = condition 
    vals=[]
    
    if type(values) in (float64, list, tuple):
        vals=values
    else:
        # Assuming that we got a pymc3 trace
        if len(var_names)<1:
            raise(Exception('If a trace is provided as values, var_names must ' +
                    'be provided as well.'))
        for v in var_names:
            vals.append(est_mode(values.get_values(v)))
    vals = array(vals)*factor       
    for i,r in enumerate(rate_names):
            label = label + '\n  $' + r +' = ' + fmt%vals[i] + '$ ' + unit 
            
    return label + '\n'
        